const Ajv = require('ajv')

const utils = require('./utils')
const { getEntryRequestSchema } = require('./schemas')
const c = require('./constants')

const ajv = new Ajv()
const validate = ajv.compile(getEntryRequestSchema())

class EntryReq {
  constructor ({ hash, location, version = undefined }) {
    this.hash = hash != null ? utils.getHashAsStr(hash) : hash // will fail later
    this.location = location
    this.version = version

    this.assertIsValid()
  }

  assertIsValid () {
    const asDict = { ...this }
    if (!validate(asDict)) {
      throw new Error(utils.ajvErrorToStr(validate.errors[0], asDict))
    }
  }

  get uniqueId () {
    return this.hash + this.location + this.version
  }

  async get (getEntryFunc) {
    return await getEntryFunc(this.toJson())
  }

  toJson () {
    const res = {
      [c.LOCATION]: this.location,
      [c.HASH]: this.hash
    }
    if (this.version != null) {
      res[c.VERSION] = this.version
    }

    return res
  }

  toStr () {
    return `${this.hash}/${this.location}`
  }
}

module.exports = EntryReq
